﻿using System;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Persistence.Interfaces {

  /// <summary>
  /// Common base class for defining a new serialization format.
  /// </summary>
  /// <typeparam name="SerialDataFormat">The type of the serial data format.</typeparam>
  [StorableClass]
  public abstract class FormatBase<SerialDataFormat> : IFormat<SerialDataFormat> where SerialDataFormat : ISerialData {

    /// <summary>
    /// Gets the format's name.
    /// </summary>
    /// <value>The format's name.</value>
    public abstract string Name { get; }
    
    /// <summary>
    /// Datatype that describes the atoms used for serialization serialization.
    /// </summary>
    public Type SerialDataType { get { return typeof(SerialDataFormat); } }

    /// <summary>
    /// Compares formats by name.
    /// </summary>
    /// <param name="f">The format.</param>
    /// <returns>wheter this object and f are equal by name.</returns>
    public bool Equals(FormatBase<SerialDataFormat> f) {
      if (f == null)
        return false;
      return f.Name == this.Name;
    }

    /// <summary>
    /// Compares foramts by name.
    /// </summary>
    /// <param name="obj">The <see cref="System.Object"/> to compare with this instance.</param>
    /// <returns>
    /// 	<c>true</c> if the specified <see cref="System.Object"/> is equal to this instance; otherwise, <c>false</c>.
    /// </returns>
    /// <exception cref="T:System.NullReferenceException">
    /// The <paramref name="obj"/> parameter is null.
    /// </exception>
    public override bool Equals(object obj) {
      FormatBase<SerialDataFormat> f = obj as FormatBase<SerialDataFormat>;
      return Equals(f);
    }

    /// <summary>
    /// Returns a hash code for this instance.
    /// </summary>
    /// <returns>
    /// A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
    /// </returns>
    public override int GetHashCode() {
      return Name.GetHashCode();
    }

  }

}